@@ -1,5 +1,5 @@
 # The contents of this file are subject to the BitTorrent Open Source License
-# Version 1.0 (the License).  You may not copy or use this file, in either
+# Version 1.1 (the License).  You may not copy or use this file, in either
 # source code or executable form, except in compliance with the License.  You
 # may obtain a copy of the License at http://www.bittorrent.com/license/.
 #
@@ -13,6 +13,7 @@
 import os
 import sys
 import gettext
+import locale
 
 # Python 2.2 doesn't have RawConfigParser
 try:
@@ -21,12 +22,15 @@ except ImportError:
     from ConfigParser import ConfigParser as RawConfigParser
 
 from ConfigParser import MissingSectionHeaderError, ParsingError
-
 from BitTorrent import parseargs
-from BitTorrent import ERROR, BTFailure
-from BitTorrent import app_name, version, is_frozen_exe, locale_root
-from __init__ import get_config_dir
+from BitTorrent import app_name, version, ERROR, BTFailure
+from BitTorrent.platform import get_config_dir, locale_root, is_frozen_exe
+from BitTorrent.defaultargs import MYTRUE
 
+TORRENT_CONFIG_FILE = 'torrent_config'
+
+alt_uiname = {'bittorrent':'btdownloadgui',
+              'maketorrent':'btmaketorrentgui',}
 
 def _read_config(filename):
     # check for bad config files (Windows corrupts them all the time)
@@ -53,6 +57,20 @@ def _read_config(filename):
     return p
 
 
+def _write_config(error_callback, filename, p):
+    try:
+        f = file(filename, 'w')
+        p.write(f)
+        f.close()
+    except Exception, e:
+        try:
+            f.close()
+        except:
+            pass
+        error_callback(ERROR, _("Could not permanently save options: ")+
+                       str(e))
+
+
 def bad_config(filename):
     base_bad_filename = filename + '.broken'
     bad_filename = base_bad_filename
@@ -67,15 +85,12 @@ def bad_config(filename):
 
 def get_config(defaults, section):
     dir_root = get_config_dir()
-    if dir_root is None and os.name == 'nt':
-        tmp_dir_root = os.path.split(sys.executable)[0]
-        if os.access(tmp_dir_root, os.R_OK|os.W_OK):
-            dir_root = tmp_dir_root
 
     if dir_root is None:
         return {}
 
     configdir = os.path.join(dir_root, '.bittorrent')
+        
     if not os.path.isdir(configdir):
         try:
             os.mkdir(configdir, 0700)
@@ -113,19 +128,44 @@ def save_ui_config(defaults, section, save_options, error_callback):
             if is_frozen_exe:
                 err_str = "You must quit %s and reinstall it. (%s)" % (app_name, err_str)
             error_callback(ERROR, err_str)
-    try:
-        f = file(filename, 'w')
-        p.write(f)
-        f.close()
-    except Exception, e:
-        try:
-            f.close()
-        except:
-            pass
-        error_callback(ERROR, _("Could not permanently save options: ")+
-                       str(e))
+    _write_config(error_callback, filename, p)
 
 
+def save_torrent_config(path, infohash, config, error_callback):
+    section = infohash.encode('hex')
+    filename = os.path.join(path, TORRENT_CONFIG_FILE)
+    p = _read_config(filename)
+    p.remove_section(section)
+    p.add_section(section)
+    for key, value in config.items():
+        p.set(section, key, value)
+    _write_config(error_callback, filename, p)
+
+def read_torrent_config(global_config, path, infohash, error_callback):
+    section = infohash.encode('hex')
+    filename = os.path.join(path, TORRENT_CONFIG_FILE)
+    p = _read_config(filename)
+    if not p.has_section(section):
+        return {}
+    else:
+        c = {}
+        for name, value in p.items(section):
+            if global_config.has_key(name):
+                t = type(global_config[name])
+                if t == bool:
+                    c[name] = value in ('1', 'True', MYTRUE, True)
+                else:
+                    c[name] = type(global_config[name])(value)
+        return c
+
+def remove_torrent_config(path, infohash, error_callback):
+    section = infohash.encode('hex')
+    filename = os.path.join(path, TORRENT_CONFIG_FILE)
+    p = _read_config(filename)
+    if p.has_section(section):
+        p.remove_section(section)
+    _write_config(error_callback, filename, p)
+
 def parse_configuration_and_args(defaults, uiname, arglist=[], minargs=0,
                                  maxargs=0):
     defconfig = dict([(name, value) for (name, value, doc) in defaults])
@@ -142,9 +182,11 @@ def parse_configuration_and_args(defaults, uiname, arglist=[], minargs=0,
                                        presets)
     datadir = config['data_dir']
     if datadir:
-        if uiname in ('btdownloadgui', 'btmaketorrentgui'):
+        if uiname in ('bittorrent', 'maketorrent'):
             values = {}
             p = _read_config(os.path.join(datadir, 'ui_config'))
+            if not p.has_section(uiname) and p.has_section(alt_uiname[uiname]):
+                uiname = alt_uiname[uiname]
             if p.has_section(uiname):
                 for name, value in p.items(uiname):
                     if name in defconfig:
@@ -153,25 +195,23 @@ def parse_configuration_and_args(defaults, uiname, arglist=[], minargs=0,
             presets.update(values)
             config, args = parseargs.parseargs(arglist, defaults, minargs,
                                                maxargs, presets)
-        rdir = os.path.join(datadir, 'resume')
-        mdir = os.path.join(datadir, 'metainfo')
-        try:
-            if not os.path.exists(datadir):
-                os.mkdir(datadir, 0700)
-            if not os.path.exists(mdir):
-                os.mkdir(mdir, 0700)
-            if not os.path.exists(rdir):
-                os.mkdir(rdir, 0700)
-        except:
-            pass
 
+        for d in ('', 'resume', 'metainfo'):
+            ddir = os.path.join(datadir, d)
+            try:
+                if not os.path.exists(ddir):
+                    os.mkdir(ddir, 0700)
+            except:
+                pass
+            
     if config['language'] != '':
         try:
             lang = gettext.translation('bittorrent', locale_root,
                                        languages=[config['language']])
             lang.install()
         except IOError:
-            raise BTFailure('Could not find translation for language "%s"' %
-                            config['language'])
+            # don't raise an error, just continue untranslated
+            sys.stderr.write('Could not find translation for language "%s"\n' %
+                             config['language'])                
     
     return config, args
